/*
	AddNew.ms
	Version: 1.71
	Created On: November 23, 2002
	Created By: Jeff Hanna
	Modified On: March 12, 2003
	Modified By: Jeff Hanna
	tested using Max 4.26

	 Copyright 2003, Lodestone Games, LLC. All Rights Reserved.
	
	Adds a new Max scene and any new bitmaps used in that scene to the version control archive.
	The files are added in a locked state, to prevent the scene from being checked out of version control by more than one user 
	at a time.
	
		
	v1.71 - Changed variable names to follow the programming guidelines.
	v1.7 - Changed name from "SaveAsToSourceControl" to "AddNew". All "Source Control" and "SC" references changed to "Version Control 
	       and "VC".  Changed function call "ConvertArray" to "ConvertArrayDOS2Unix" to reflect the change made in LSUtilLib
	v1.6 - Fixed a bug with a misnamed variable that was breaking the script.
	v1.5 - Copyright information added.
	v1.4 - Switched from storing strMaxFileToEdit and strStartMapsFile in the FileProperties page to storing them in the persistent 
	       global variables, "pg_strMaxFilepath" and, "pg_strMapsFilepath".
	v1.3 - Changed the function call "ConvertArrays" to "ConvertArray" to reflect the change made in LSUtilLb.
	v1.2 - Initial release.
*/

(
	--LOCAL VARIABLES
	-----------------
	local strAddNewVersion = "1.71" as string	-- Version number for display in the Max Listener window
	local bExit = false							-- Boolean to control whether the script should exit or not.
	local bReOpen = false						-- Boolean to indicate whether the user wants to immediately re-open the file.
	
	aAllMaps = #()
	aAllPathnames = #()
	aFilesToSubmit = #()
	aNewPathnames = #()
	aOtherMaps = #()

		
	--GLOBAL VARIABLES
	------------------
	

	--INCLUDED FILES
	----------------
	include "$scripts\\Lodestone\\LSUtilLib.ms"
	include "$scripts\\Lodestone\\VersionControl\\P4\\P4Lib.ms"
	

	--MAIN LOOP
	-----------
	-- print version information to the listener. Useful if a bug-report needs to be sent.
	print ("AddNew, v" + strAddNewVersion + " loaded.")
	 
	-- This script is executed as a do-while loop so that it can be exited at any time 
	do
	(
		-- Check the persistent global variable stored in the file.
		if pg_strMapsFilepath != undefined then
		(
				-- If the variable exits, then this file is already in source control. It can't be added a second time.
				-- Alert the user and exit the script.
				messagebox "It appears that this scene already exists in the version control archive.\nThis scene cannot be added a second time." title:"Version Control"
			
				bExit = true
				exit
		)
		else
		(
			-- If the variable doesn't exist, then the file is new and the variable needs to be set.
			persistent global pg_strMaxFilepath = getMaxSaveFilename types:"3ds max(*.max)|*.max|All|*.*|"
			
			-- Check to see if the persistent global map path variable is undefined
			if pg_strMaxFilepath != undefined then
			(
				-- If it is, then set it.
				persistent global pg_strMapsFilepath = ("$temp\\" + (getFilenameFile pg_strMaxFilePath) + "_StartMaps.txt")
				
				-- The user needs to enter a description of the changes and decide if they want to close the scene.
				createDialog rolloutGetDescription width:250 modal:true
				bReOpen = querybox "Would you like to reopen the scene and continue to work on it?" title:"Version Control"
			
				-- Get the max file name from the persistent global variable and save the file.
				saveMaxFile pg_strMaxFilepath
				
				-- Traverse the node tree in the scene and gather all maps used.
				-- Find both maps assigned to materials and those that aren't.
				aAllMaps = GetSceneMaps()
				aOtherMaps = GetOtherMaps()
				aAllMaps = aAllMaps + aOtherMaps
				
				-- Get the bitmap file paths from the maps and then remove any duplicate entries.
				aNewPathnames = GetBitmapsFromMaps aAllMaps
				RemoveDuplicates aNewPathnames
				
				-- The Max scene wouldn't be included in the Submit function if it's path and name weren't retrieved and added to the array.
				append aNewPathnames pg_strMaxFilepath
				
				-- Add all new bitmaps to the source control archive.
				VCAdd aNewPathnames

				-- Forcing a revert on all files will cause the source control system to revert any file that is unchanged
				VCRevertUnchanged aNewPathnames

				-- Convert the path names to Unix style relative pathing.
				aNewPathnames = ConvertArrayDOS2UNIX aNewPathnames

				-- Check the changelist after the revert and build an array, "aFilesToSubmit" from what's left.
				aFilesToSubmit = VCGetOpenedFiles aNewPathnames
				
				-- Check to see if the user wanted to continue editng the file
				if bReOpen == false then
				(
					-- If not, then submit the files.
					VCSubmit aFilesToSubmit

					-- Clean up any temporary files on the HD.
					deleteFile pg_strMapsFilePath
					deleteFile "$temp\\AddFiles.txt"
					deleteFile "$temp\\OpenedFiles.txt"
					deleteFile "$temp\\RevertFiles.txt"
					deleteFile "$temp\\SubmitFiles.txt"
			
					--Resetting Max leaves global variables around. These commands undeclare them.
					pg_strMapsFilepath = undefined
					pg_strMaxFilepath = undefined
					
					-- Reset the Max scene to prevent and further editing of the submitted file.
					resetMaxFile #noPrompt
					
					bExit = true
					exit
				)
				else
				(
					-- If they did want to continue editing, submit the files and 
					-- immediately check them out again from the source control archive.
					VCSubmitAndReopen aFilesToSubmit
					
					-- Force a synchronization with the source control system, just to be on the safe side.
					VCSyncAndCheckOut aAllPathnames pg_strMapsFilepath
					
					-- Clean up any temporary files on the HD.
					deleteFile "$temp\\AddFiles.txt"
					deleteFile "$temp\\OpenedFiles.txt"
					deleteFile "$temp\\RevertFiles.txt"
					deleteFile "$temp\\SubmitFiles.txt"
			
			bExit = true
					exit
				)
			)
			else
			(
				bExit = true
				exit
			)
		)
	)
	while bExit == false
)