/*
	P4Lib.ms
	Version: 1.91
	Created On: December 01, 2002
	Created By: Jeff Hanna
	Modified On: March 12, 2003
	Modified By: Jeff Hanna
	tested using Max 4.26

	 Copyright 2003, Lodestone Games, LLC. All Rights Reserved.
	
	A set of scripts that interface the Version Control MaxScript package to Perforce.
	If the Version Control MaxScripts need to be ported to another versioning system, this file is where the majority of the changes
	will occur.
	
	
	v1.91 - Changed all global string variables to local string variables. Renamed variabled prefixes to match coding guidelines.
	v1.9 - Changed all instances of "SC" to "VC" to reflect the renaming of the package to "Version Control"
	v1.8 - SCAdd function adds binary files in exclusive mode now. Exclusive mode files can only be checked out by one person at a time.
	v1.7 - Added SCGetFileStats function to prevent file from being checked out twice.
	v1.6 - Copyright information added.
	v1.5 - Initial release.
*/

(
	--LOCAL VARIABLES
	-----------------
	local strVersion = "1.91" as string
	
	local strAddFile = "$temp\\AddFiles.txt" as string
	local strRevertFile = "$temp\\RevertFiles.txt" as string
	local strSubmitFile = "$temp\\SubmitFiles.txt" as string
	local strOpenedFiles = "$temp\\OpenedFiles.txt" as string
	local strDescription = "" as string
	local strStartMapsFile = "" as string
	
	
	--GLOBAL VARIABLES
	------------------
		
	
	--ROLLOUTS
	----------
	-- The change lists are more useful with user-entered descriptions about what was done during the edit session.
	rollout rltGetDescription "Version Control"
	(
		label lblWhatFor "Please enter a change description for this scene."
		editText edtUserEntry
		button btnOK "OK" enabled:false
		
		on edtUserEntry changed strEntry do
		(
			strDescription = strEntry
			btnOK.enabled = true
		)
		on btnOK pressed do
		(
			destroyDialog rltGetDescription
		)
	)
	-------------------------------------------------------------------------------------------------------------------------
	
	
	--TEXT FILE CREATION FUNCTIONS
	------------------------------
	-- The "Submit" function takes a specially formed text file to be read from standard input.
	fn WriteSubmitTemplate aInputArray strFilePath =
	(
		local FileOut = createFile strFilePath
		
		format "Change:\tnew\n\n" to:FileOut
		format "Status:\tnew\n\n" to:FileOut
		format "Description:\n" to:FileOut
		format "\t%\n\n" strDescription to:FileOut			
		format "Files:\n\n" to:FileOut
		
		for i in aInputArray do
		(
			format "\t%\n" i to:FileOut
		)
		close FileOut
	)
	-------------------------------------------------------------------------------------------------------------------------	
	
	
	--SOURCE CONTROL COMMAND FUNCTIONS
	----------------------------------
	-- Check a max scene file against the depot to see if someone has it checked out.
	-- Takes the path file of the max scene as input.
	-- Returns a boolean, True if the file is checked out by another user, false if it isn't.
	fn VCGetFileStats strMaxSceneFile =
	(
		DOSCommand ("p4 fstat " + strMaxSceneFile + " > %temp%\VC_fstat.txt")
	
		local fileIn = openFile "$temp\\VC_fstat.txt" mode:"r"
	
		if skipTostring fileIn "... ... " != undefined then
		 (
			 local strFstatData = readDelimitedString fileIn "@"
			 
			 close fileIn
			 
			 aStringPieces = filterString strFstatData " "
			 
			 local strUserName = (aStringPieces[aStringPieces.count])
			 
			 messagebox ("This file is already checked out by " + strUserName + ".\n\nA Max scene can not be checked out by two people simultaneously.\n\nIf you need to work on this file please ask " + strUserName + " to submit\nto submit\the changes or revert the file.")
			 
			 deleteFile "$temp\\VC_fstat.txt"
			 
			 return true
		 )
		 else
		 (
			 close fileIn
			 
			 deleteFile "$temp\\VC_fstat.txt"
			 
			 return false
		 )
	)
	-------------------------------------------------------------------------------------------------------------------------
	
	-- Pass a list of requested files to the source control system so they can be sync'd to their latest versions.
	fn VCSync aFilesToSync strTextFile =
	(
		WriteFile aFilesToSync strStartMapsFile
		
		strDOSSyncFile = DOSTempPath strTextFile 1 5
		DOSCommand ("p4 -x " + strDOSSyncFile + " sync")
	)
	-------------------------------------------------------------------------------------------------------------------------

	-- Pass a list of requested files to the source control system so they can be sync'd to their latest versions and opened for edit.
	fn VCSyncAndCheckOut aFilesToCheckOut strTextFile =
	(
		WriteFile aFilesToCheckOut strTextFile
		
		strDOSEditFile = DOSTempPath strTextFile 1 5
		DOSCommand ("p4 -x " + strDOSEditFile + " sync")
		DOSCommand ("p4 -x " + strDOSEditFile + " edit")
	)		
	-------------------------------------------------------------------------------------------------------------------------
	
	-- Pass a list of new files to the source control system so they can be added.
	fn VCAdd aFilesToAdd =
	(
		WriteFile aFilesToAdd strAddFile
		
		strDOSAddFile = DOSTempPath strAddFile 1 5
		DOSCommand ("P4 -x " + strDOSAddFile + " add -t binary+l")
	)	
	-------------------------------------------------------------------------------------------------------------------------
	
	-- Pass a list of unchanged files to the source control system so they can be reverted. Best used at the end of an editing
	-- session before a submit happens.
	fn VCRevertUnchanged aFilesToRevert =
	(
		WriteFile aFilesToRevert strRevertFile
		
		strDOSRevertFile = DOSTempPath strRevertFile 1 5
		DOSCommand ("P4 -x " + strDOSRevertFile + " revert -a")
	)
	-------------------------------------------------------------------------------------------------------------------------
	
	-- Discard all changes and revert all opened files to their latest version.
	fn VCRevertAll aFilesToRevert =
	(
		WriteFile aFilesToRevert strRevertFile
		
		strDOSRevertFile = DOSTempPath strRevertFile 1 5

		DOSCommand ("P4 -x " + strDOSRevertFile + " revert")
	)		
	-------------------------------------------------------------------------------------------------------------------------
	
	-- Pass a list of opened and changed files to the source control system so the changes can be submitted.
	fn VCSubmit aFilesToSubmit =
	(
		WriteSubmitTemplate aFilesToSubmit strSubmitFile
		
		strDOSSubmitFile = DOSTempPath strSubmitFile 1 5
		DOSCommand ("type " + strDOSSubmitFile + " | p4 submit -i")	
	)	
	-------------------------------------------------------------------------------------------------------------------------
	
	-- Pass a list of opened and changed files to the source control system so the changes can be submitted. 
	-- Reopen the list of files for edit after the submit has completed.
	fn VCSubmitAndReopen aFilesToSubmit =
	(
		WriteSubmitTemplate aFilesToSubmit strSubmitFile
		
		strDOSSubmitFile = DOSTempPath strSubmitFile 1 5
		DOSCommand ("type " + strDOSSubmitFile + " | p4 submit -i -r")
	)	
	-------------------------------------------------------------------------------------------------------------------------
	
	-- A list of the remaining opened files needs to be gathered so that the program knows what to submit.
	-- Reading the .txt file into an array and comparing it against the array of all pathnames used in
	-- the scene will yield just the files that need to be submitted.
	fn VCGetOpenedFiles aInputArray = 
	(
		local aReturnArray = #()
		
		DOSCommand ("P4 opened > %temp%\OpenedFiles.txt")
		
		local FileIn = openFile strOpenedFiles
		while not eof FileIn do
		(
			local strTextLine = (readLine FileIn) as string
			aOpenedFiles = filterString strTextLine "#"
			
			for i in aOpenedFiles do
			(
				local iIndex = findItem aInputArray i
				
				if iIndex != 0 then
				(
					append aReturnArray i
				)
			)	
		)
		close FileIn
		return aReturnArray
	)
	-------------------------------------------------------------------------------------------------------------------------
	
	
	-- print version information to the listener. Useful if a bug-report needs to be sent.
	print ("P4Lib, v" + strVersion + " loaded.")
)